@@ -17,13 +17,13 @@ from django.views.generic.edit import CreateView, UpdateView
 from django.views.generic import ListView
 from django import forms
 
+import warnings
 import datetime
 import inspect
 import time
 import re
 from decimal import Decimal
 from numbers import Number
-from types import BooleanType
 import copy
 from dateutil import parser
 
@@ -46,6 +46,8 @@ class ReportEditForm(forms.ModelForm):
 class DisplayFieldForm(forms.ModelForm):
     class Meta:
         model = DisplayField
+        fields = ('name', 'path', 'path_verbose', 'field_verbose', 'field', 'position',
+                  'width', 'total', 'sort', 'aggregate', 'group', 'display_format')
         widgets = {
             'path': forms.HiddenInput(),
             'path_verbose': forms.TextInput(attrs={'readonly':'readonly'}),
@@ -60,6 +62,8 @@ class DisplayFieldForm(forms.ModelForm):
 class FilterFieldForm(forms.ModelForm):
     class Meta:
         model = FilterField
+        fields = ('path', 'path_verbose', 'field_verbose', 'field', 'filter_type',
+                 'filter_value', 'filter_value2', 'exclude', 'position')
         widgets = {
             'path': forms.HiddenInput(),
             'path_verbose': forms.TextInput(attrs={'readonly':'readonly'}),
@@ -123,8 +127,11 @@ def get_properties_from_model(model_class):
     properties = []
     attr_names = [name for (name, value) in inspect.getmembers(model_class, isprop)]
     for attr_name in attr_names:
-        properties.append(dict(label=attr_name, name=attr_name.strip('_').replace('_',' ')))
-    return sorted(properties)
+        if attr_name.endswith('pk'):
+            attr_names.remove(attr_name)
+        else:
+            properties.append(dict(label=attr_name, name=attr_name.strip('_').replace('_',' ')))
+    return sorted(properties, key=lambda k: k['label'])
 
 def filter_property(filter_field, value):
     filter_type = filter_field.filter_type
@@ -232,6 +239,27 @@ def ajax_get_related(request):
         'path_verbose': path_verbose,
     }, RequestContext(request, {}),)
 
+def fieldset_string_to_field(fieldset_dict, model):
+    if isinstance(fieldset_dict['fields'], tuple):
+        fieldset_dict['fields'] = list(fieldset_dict['fields'])
+    i = 0
+    for dict_field in fieldset_dict['fields']:
+        if isinstance(dict_field, basestring):
+            fieldset_dict['fields'][i] = model._meta.get_field_by_name(dict_field)[0]
+        elif isinstance(dict_field, list) or isinstance(dict_field, tuple):
+            dict_field[1]['recursive'] = True
+            fieldset_string_to_field(dict_field[1], model)
+        i += 1
+
+def get_fieldsets(model):
+    """ fieldsets are optional, they are defined in the Model.
+    """
+    fieldsets = getattr(model, 'report_builder_fieldsets', None)
+    if fieldsets:
+        for fieldset_name, fieldset_dict in model.report_builder_fieldsets:
+            fieldset_string_to_field(fieldset_dict, model)
+    return fieldsets
+
 @staff_member_required
 def ajax_get_fields(request):
     """ Get fields and properties for a particular model
@@ -244,10 +272,12 @@ def ajax_get_fields(request):
     custom_fields = get_custom_fields_from_model(model)
     root_model = model.__name__.lower()
     app_label = model._meta.app_label
+    fieldsets = get_fieldsets(model)
 
     if field_name == '':
         return render_to_response('report_builder/report_form_fields_li.html', {
             'fields': get_direct_fields_from_model(model),
+            'fieldsets': fieldsets,
             'properties': properties,
             'custom_fields': custom_fields,
             'root_model': root_model,
@@ -290,6 +320,7 @@ def ajax_get_fields(request):
     
     return render_to_response('report_builder/report_form_fields_li.html', {
         'fields': fields,
+        'fieldsets': fieldsets,
         'custom_fields': custom_fields,
         'properties': properties,
         'path': path,
@@ -339,11 +370,12 @@ def report_to_list(report, user, preview=False, queryset=None):
         objects = report.add_aggregates(queryset)
     else:
         try:
-            objects = report.get_query()
-        except exceptions.ValidationError, e:
+            objects, query_message = report.get_query()
+            message += query_message
+        except exceptions.ValidationError as e:
             message += "Validation Error: {0!s}. Something may be wrong with the report's filters.".format(e)
             return [], message
-        except ValueError, e:
+        except ValueError as e:
             message += "Value Error: {0!s}. Something may be wrong with the report's filters. For example it may be expecting a number but received a character.".format(e)
             return [], message
 
@@ -402,7 +434,7 @@ def report_to_list(report, user, preview=False, queryset=None):
             def increment_total(display_field_key, display_totals, val):
                 if display_totals.has_key(display_field_key):
                     # Booleans are Numbers - blah
-                    if isinstance(val, Number) and not isinstance(val, BooleanType):
+                    if isinstance(val, Number) and not isinstance(val, bool):
                         # do decimal math for all numbers
                         display_totals[display_field_key]['val'] += Decimal(str(val))
                     else:
@@ -576,8 +608,14 @@ def report_to_list(report, user, preview=False, queryset=None):
                         value = Decimal(display_totals_row[df.position-1])
                     except:
                         value = display_totals_row[df.position-1]
-                    display_totals_row[df.position-1] = df.display_format.string.\
-                        format(value)
+                    # Fall back to original value if format string and value
+                    # aren't compatible, e.g. a numerically-oriented format
+                    # string with value which is not numeric.
+                    try:
+                        value = df.display_format.string.format(value)
+                    except ValueError:
+                        pass
+                    display_totals_row[df.position-1] = value
 
         if display_totals:
             values_and_properties_list = (
@@ -588,7 +626,8 @@ def report_to_list(report, user, preview=False, queryset=None):
 
                 
 
-    except exceptions.FieldError:
+    except exceptions.FieldError as e:
+        warnings.warn('Error {0}'.format(str(e)))
         message += "Field Error. If you are using the report builder then you found a bug!"
         message += "If you made this in admin, then you probably did something wrong."
         values_and_properties_list = None
@@ -655,6 +694,7 @@ class ReportUpdateView(UpdateView):
         
         ctx['related_fields'] = relation_fields
         ctx['fields'] = direct_fields
+        ctx['fieldsets'] = get_fieldsets(model_class)
         ctx['custom_fields'] = custom_fields
         ctx['properties'] = properties
         ctx['model_ct'] = model_ct
@@ -695,7 +735,7 @@ def download_xlsx(request, pk, queryset=None):
     
     wb = Workbook()
     ws = wb.worksheets[0]
-    ws.title = report.name[:30]
+    ws.title = re.sub(r'\W+', '', report.name)[:30]
     filename = re.sub(r'\W+', '', report.name) + '.xlsx'
     
     i = 0
